@@ -1,4 +1,4 @@
-from typing import Dict, Optional, Tuple
+from typing import Dict, List, Optional, Tuple, Union
 
 import torch
 import torch.nn as nn
@@ -207,22 +207,50 @@ class StarDistUnet(BaseMultiTaskSegModel):
         if enc_freeze:
             self.freeze_encoder()
 
-    def forward(self, x: torch.Tensor) -> Dict[str, torch.Tensor]:
-        """Forward pass of Stardist."""
-        self._check_input_shape(x)
+    def forward(
+        self,
+        x: torch.Tensor,
+        return_feats: bool = False,
+    ) -> Union[
+        Dict[str, torch.Tensor],
+        Tuple[
+            List[torch.Tensor],
+            Dict[str, torch.Tensor],
+            Dict[str, torch.Tensor],
+        ],
+    ]:
+        """Forward pass of Stardist.
 
-        feats = self.encoder(x)
+        Parameters
+        ----------
+            x : torch.Tensor
+                Input image batch. Shape: (B, C, H, W).
+            return_feats : bool, default=False
+                If True, encoder, decoder, and head outputs will all be returned
 
-        style = None
-        if self.make_style is not None:
-            style = self.make_style(feats[0])
+        Returns
+        -------
+        Union[
+            Dict[str, torch.Tensor],
+            Tuple[
+                List[torch.Tensor],
+                Dict[str, torch.Tensor],
+                Dict[str, torch.Tensor],
+            ],
+        ]:
+            Dictionary mapping of output names to outputs or if `return_feats == True`
+            returns also the encoder features in a list, decoder features as a dict
+            mapping decoder names to outputs and the final head outputs dict.
+        """
+        feats, dec_feats = self.forward_features(x)
 
-        dec_feats = self.forward_dec_features(feats, style)
         # Extra convs after decoders
         for e in self.extra_convs.keys():
             for extra_conv in self.extra_convs[e].keys():
                 k = self.aux_key if extra_conv not in dec_feats.keys() else extra_conv
-                dec_feats[extra_conv] = self[f"{extra_conv}_features"](dec_feats[k])
+                dec_feats[extra_conv] = [
+                    self[f"{extra_conv}_features"](dec_feats[k][-1])
+                ]  # use last decoder feat
 
         # seg heads
         for decoder_name in self.heads.keys():
@@ -233,6 +261,9 @@ class StarDistUnet(BaseMultiTaskSegModel):
 
         out = self.forward_heads(dec_feats)
 
+        if return_feats:
+            return feats, dec_feats, out
+
         return out
 
 
